--Command.System.Strict()

local addon, shared = ...

local data = {
	complete = {},
	playerAlliance = "",
	initialized = false,
	guardian = { dwarf = true, highelf = true, mathosian = true }
}

-- ////////////////////////// Helpers /////////////////////////////////////////

-- Inspect.Unit.Detail("player").alliance may change temporarily (when player
-- is a mercenary). It appears safer to obtain the alliance by checking the
-- race.
local function getPlayerAlliance()
	local race = Inspect.Unit.Detail("player").race
	return data.guardian[race] and "guardian" or "defiant"
end

-- The quest list from Inspect.Quest.Complete() appears to be cached and never
-- changes once you read it. So it should be safe to store and modify it.
local function checkData()
	if not data.initialized then
		data.complete = Inspect.Quest.Complete()
		data.playerAlliance = getPlayerAlliance()
		data.initialized = true
	end
end

-- There are quests which have a faction like "Order of Mathos". These should
-- probably be allowed for both player factions!
local function isQuestForAll(faction)
	local otherAlliance = (data.playerAlliance == "guardian") and "defiant" or "guardian"
	return string.find(faction, data.playerAlliance) == nil and string.find(faction, otherAlliance) == nil
end

local function printDetails(questData)
	if questData.scene == nil then
		print(questData.name)
	else
		print(string.format("%s (%s)", questData.name, questData.scene))
	end
end

-- ////////////////////////// Event handlers //////////////////////////////////

local function complete(handle, elements)
	for k,v in pairs(elements) do
		local fullId = string.sub(k, 1, 9) .. "xxxxxxxx"
		data.complete[fullId] = true
	end
end

-- ////////////////////////// Slash command handlers //////////////////////////

local function search(handle, args)
	if #args == 0 then
		print(string.format("TheFinalQuest %s\nUsage: /tfq <zone>", addon.toc.Version))
	else
		checkData()
		local zoneCount = 0
		for zoneName, zoneQuests in pairs(shared.quests) do
			if string.find(string.lower(zoneName), string.lower(args)) ~= nil then
				local questCount = 0
				for questId, questData in pairs(zoneQuests) do
					local fullId = questId .. "xxxxxxxx"
					local secondaryId = "qFFFFFFFF" .. string.sub(questId, 2) .. "xxxxxxxx"
					if data.complete[fullId] == nil and data.complete[secondaryId] == nil then
						if questData.faction == nil then
							printDetails(questData)
							questCount = questCount + 1
						else
							local faction = string.lower(questData.faction)
							if string.find(faction, data.playerAlliance) ~= nil or isQuestForAll(faction) then
								printDetails(questData)
								questCount = questCount + 1
							end
						end
					end
				end
				print(string.format("*** Found %d missing quest(s) for %s!", questCount, zoneName))
				zoneCount = zoneCount + 1
			end
		end
		if zoneCount == 0 then
			print(string.format("No matching zones found for '%s'!", args))
		end
	end
end

-- ////////////////////////// Addon initialisation ////////////////////////////

Command.Event.Attach(Event.Quest.Complete, complete, "Quest.Complete callback")

Command.Event.Attach(Command.Slash.Register("tfq"), search, "Slash command 'tfq'")

--print("Initialized")
